html { font-family: sans-serif; line-height: 1.5; }
main { max-width: 800px; margin: 0 auto; padding: 1em; }
p { overflow-wrap: break-word; }
img { max-width: 100%; }
#title-block-header { display: none; }
.level1 { display: none; padding: 1em 1.5em; }
.level1:target { display: block; }
ul.slipbox-list { list-style: none; }
